@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@ open Source
 
 exception Error
 
+(* This is an internal operator. *)
 class insert_metadata ~kind source =
   object (self)
     inherit operator ~name:"insert_metadata" kind [source]
@@ -47,11 +48,12 @@ class insert_metadata ~kind source =
     method private add_metadata frame pos =
       Tutils.mutexify lock_m
         (fun () ->
-          match metadata with
-            | None -> ()
-            | Some m ->
+          (match (metadata, Frame.get_metadata frame pos) with
+            | Some m, None ->
                 metadata <- None;
-                Frame.set_metadata frame pos m)
+                Frame.set_metadata frame pos m
+            | _ -> ());
+          metadata <- None)
         ()
 
     method private insert_track =
