@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -68,7 +68,7 @@ class prepend ~kind ~merge source f =
             let peek_pcm = AFrame.pcm peek in
             let peekpos = AFrame.size () - 1 in
             for i = 0 to Array.length pcm - 1 do
-              pcm.(i).{p} <- peek_pcm.(i).{peekpos}
+              pcm.(i).(p) <- peek_pcm.(i).(peekpos)
             done;
             begin
               match AFrame.get_metadata peek peekpos with
