@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,37 +23,36 @@
 let log = Log.make ["playlist"; "basic"]
 let split_lines buf = Pcre.split ~pat:"[\r\n]+" buf
 
-let test_text s =
-  match Configure.data_mime with
-    | None -> ()
-    | Some get_mime ->
-        let mime = get_mime s in
-        if not (Pcre.pmatch ~pat:"text/.*" mime) then (
-          log#important "Wrong mime type %s for playlist!" mime;
-
-          (* TODO this shouldn't be an assert false, it can happen *)
-          assert false)
-
 let parse_extinf s =
   try
-    let rex = Pcre.regexp "#EXTINF:(\\d+),(.*)" in
+    let rex = Pcre.regexp "#EXTINF:(\\d*)\\s*([^\\s]*),(.*)" in
     let sub = Pcre.exec ~rex s in
-    let duration = Pcre.get_substring sub 1 in
-    let song = Pcre.get_substring sub 2 in
-    let lines = Pcre.split ~pat:" - " song in
+    let meta =
+      match Pcre.get_substring sub 1 with
+        | "" -> []
+        | duration -> [("extinf_duration", duration)]
+    in
+    let meta =
+      meta
+      @
+      match Pcre.get_substring sub 2 with
+        | "" -> []
+        | key_values -> (
+            try fst (Annotate.parse (key_values ^ ":foo")) with _ -> [])
+    in
+    let song = Pcre.get_substring sub 3 in
+    let lines = Pcre.split ~pat:"\\s*-\\s*" song in
+    meta
+    @
     match lines with
+      | [""; song] -> [("song", String.trim song)]
       | [artist; title] ->
-          [
-            ("extinf_duration", duration);
-            ("artist", String.trim artist);
-            ("title", String.trim title);
-          ]
-      | _ -> [("extinf_duration", duration); ("song", String.trim song)]
+          [("artist", String.trim artist); ("title", String.trim title)]
+      | _ -> [("song", String.trim song)]
   with Not_found -> []
 
 (* This parser cannot detect the format !! *)
 let parse_mpegurl ?pwd string =
-  test_text string;
   let lines = List.filter (fun x -> x <> "") (split_lines string) in
   let is_info line = Pcre.pmatch ~pat:"^#EXTINF" line in
   let skip_line line = line.[0] == '#' in
@@ -70,7 +69,6 @@ let parse_mpegurl ?pwd string =
   get_urls [] lines
 
 let parse_scpls ?pwd string =
-  test_text string;
   let string = Pcre.replace ~pat:"#[^\\r\\n]*[\\n\\r]+" string in
   (* Format check, raise Not_found if invalid *)
   ignore
@@ -204,7 +202,6 @@ let parse_tracks index lines =
   parse [] track lines
 
 let parse_cue ?pwd string =
-  test_text string;
   let strings = split_lines string in
   let strings =
     List.map
