@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -56,10 +56,6 @@ type namespace = string list
 (** Get a string representation of a namespace. *)
 val to_string : namespace -> string
 
-(** Register an operator with given identifier and kind. This is only used in
-   the "list" telnet command.  *)
-val register_op : string -> string -> unit
-
 (** Specialized implementation of conditions to use in server commands. *)
 type condition = {
   wait : (unit -> string) -> unit;
@@ -94,3 +90,6 @@ val remove : ns:namespace -> string -> unit
 (** [exec command] executes the [command], returns its result.
   * @raise Not_found if the command does not exist.  *)
 val exec : string -> string
+
+(** Start server and telnet if enabled. *)
+val start : unit -> unit
