@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 external force_locale : unit -> unit = "liquidsoap_set_locale" [@@noalloc]
 
 (** Get page size. *)
-external pagesize : unit -> int = "liquidsoao_get_pagesize"
+external pagesize : unit -> int = "liquidsoap_get_pagesize"
   [@@noalloc]
 
 let pagesize = pagesize ()
@@ -127,17 +127,27 @@ let randomize a =
       permute i (i + Random.int (l - i))
     done
 
-(* Generic escaping function *)
-let escape ~special_char ~next ~escape_char f s =
-  let out = Format.pp_print_string f in
-  let len = String.length s in
-  let rec f pos =
-    let new_pos = next s (pos : int) in
-    let s = String.sub s pos (new_pos - pos) in
-    if special_char s then out (escape_char s) else out s;
-    if new_pos < len then f new_pos
+(* Generic escaping function. Returns a list of escaping substitutions. *)
+let escape ~special_char ~next ~escape_char s =
+  let orig_len = String.length s in
+  let rec f (segments, total_len) pos =
+    let next_pos = next s (pos : int) in
+    let len = next_pos - pos in
+    let ret =
+      if special_char s pos len then (
+        let s = escape_char s pos len in
+        let len = String.length s in
+        (`Subst (s, len) :: segments, total_len + len))
+      else
+        ( (match segments with
+            | `Orig (pos, s_len) :: segments ->
+                `Orig (pos, s_len + len) :: segments
+            | segments -> `Orig (pos, len) :: segments),
+          total_len + len )
+    in
+    if next_pos < orig_len then f ret next_pos else ret
   in
-  if len > 0 then f 0
+  if orig_len > 0 then f ([], 0) 0 else ([], 0)
 
 let utf8_next s i =
   match s.[i] with
@@ -149,18 +159,21 @@ let utf8_next s i =
 
 (* Return the utf8 char code of the first utf8 character in the given
    string. *)
-let utf8_char_code s =
-  match s.[0] with
+let utf8_char_code s pos len =
+  if len < 1 then failwith "invalid utf8";
+  match s.[pos] with
     | '\000' .. '\127' as c -> Char.code c
     | '\192' .. '\223' as c ->
+        if len < 2 then failwith "invalid utf8";
         let n1 = Char.code c in
-        let n2 = Char.code s.[1] in
+        let n2 = Char.code s.[pos + 1] in
         if n2 lsr 6 != 0b10 then failwith "invalid utf8";
         ((n1 land 0x1f) lsl 6) lor (n2 land 0x3f)
     | '\224' .. '\239' as c ->
+        if len < 3 then failwith "invalid utf8";
         let n1 = Char.code c in
-        let n2 = Char.code s.[1] in
-        let n3 = Char.code s.[2] in
+        let n2 = Char.code s.[pos + 1] in
+        let n3 = Char.code s.[pos + 2] in
         if n2 lsr 6 != 0b10 || n3 lsr 6 != 0b10 then failwith "invalid utf8";
         let p =
           ((n1 land 0x0f) lsl 12) lor ((n2 land 0x3f) lsl 6) lor (n3 land 0x3f)
@@ -168,10 +181,11 @@ let utf8_char_code s =
         if p >= 0xd800 && p <= 0xdf00 then failwith "invalid utf8";
         p
     | '\240' .. '\247' as c ->
+        if len < 4 then failwith "invalid utf8";
         let n1 = Char.code c in
-        let n2 = Char.code s.[1] in
-        let n3 = Char.code s.[2] in
-        let n4 = Char.code s.[3] in
+        let n2 = Char.code s.[pos + 1] in
+        let n3 = Char.code s.[pos + 2] in
+        let n4 = Char.code s.[pos + 3] in
         if n2 lsr 6 != 0b10 || n3 lsr 6 != 0b10 || n4 lsr 6 != 0b10 then
           failwith "invalid utf8";
         ((n1 land 0x07) lsl 18)
@@ -182,44 +196,52 @@ let utf8_char_code s =
 
 (* End of Extlib code *)
 
-let ascii_special_char = function
-  | "'" | "\"" | "\\"
-  (* DEL *)
-  | "\x7F" ->
-      true
-  (* Control chars *)
-  | s when String.length s = 1 && Char.code s.[0] <= 0x1F -> true
-  | s when String.length s = 1 && Char.code s.[0] > 0x7E -> true
-  | _ -> false
+let ascii_special_char s pos len =
+  match (s.[pos], len) with
+    | '\'', 1
+    | '"', 1
+    | '\\', 1
+    (* DEL *)
+    | '\x7F', 1 ->
+        true
+    (* Control chars *)
+    | c, 1 when Char.code c <= 0x1F -> true
+    | c, 1 when Char.code c > 0x7E -> true
+    | _ -> false
 
-let utf8_special_char s = String.length s = 1 && ascii_special_char s
+let utf8_special_char s ofs len = len = 1 && ascii_special_char s ofs len
 let ascii_next _ i = i + 1
 
-let escape_char ~escape_fun = function
-  | "\b" -> "\\b"
-  | "\n" -> "\\n"
-  | "\r" -> "\\r"
-  | "\t" -> "\\t"
-  | "\\" -> "\\\\"
-  | "\"" -> "\\\""
-  | "\'" -> "\\'"
-  | c -> escape_fun c
+let escape_char ~escape_fun s pos len =
+  match (s.[pos], len) with
+    | '\b', 1 -> "\\b"
+    | '\n', 1 -> "\\n"
+    | '\r', 1 -> "\\r"
+    | '\t', 1 -> "\\t"
+    | '\\', 1 -> "\\\\"
+    | '"', 1 -> "\\\""
+    | '\'', 1 -> "\\'"
+    | _ -> escape_fun s pos len
 
 let escape_utf8_char =
-  let utf8_char_code s =
-    try utf8_char_code s with _ -> Uchar.to_int Uchar.rep
+  let utf8_char_code s pos len =
+    try utf8_char_code s pos len with _ -> Uchar.to_int Uchar.rep
   in
-  escape_char ~escape_fun:(fun s -> Printf.sprintf "\\u%04X" (utf8_char_code s))
+  escape_char ~escape_fun:(fun s pos len ->
+      Printf.sprintf "\\u%04X" (utf8_char_code s pos len))
 
 let escape_utf8_formatter ?(special_char = utf8_special_char) =
   escape ~special_char ~escape_char:escape_utf8_char ~next:utf8_next
 
 let escape_hex_char =
-  escape_char ~escape_fun:(fun s ->
-      Printf.sprintf "\\x%02X" (int_of_char s.[0]))
+  escape_char ~escape_fun:(fun s pos len ->
+      assert (len = 1);
+      Printf.sprintf "\\x%02X" (int_of_char s.[pos]))
 
 let escape_octal_char =
-  escape_char ~escape_fun:(fun s -> Printf.sprintf "\\%03o" (int_of_char s.[0]))
+  escape_char ~escape_fun:(fun s pos len ->
+      assert (len = 1);
+      Printf.sprintf "\\%03o" (int_of_char s.[pos]))
 
 (* We use the \xhh syntax to make sure the resulting string is also consistently readable in
    OCaml. \nnn can be tricky since they are octal sequences in liquidsoap and digital sequences
@@ -227,7 +249,27 @@ let escape_octal_char =
 let escape_ascii_formatter ?(special_char = ascii_special_char) =
   escape ~special_char ~escape_char:escape_hex_char ~next:ascii_next
 
-let escape_string = Format.asprintf "%a"
+let has_subst = List.exists (function `Subst _ -> true | _ -> false)
+
+let escape_string escape s =
+  let segments, len = escape s in
+  if not (has_subst segments) then s
+  else (
+    let b = Bytes.create len in
+    ignore
+      (List.fold_left
+         (fun b_pos segment ->
+           match segment with
+             | `Orig (s_pos, len) ->
+                 let b_pos = b_pos - len in
+                 Bytes.blit_string s s_pos b b_pos len;
+                 b_pos
+             | `Subst (subst, len) ->
+                 let b_pos = b_pos - len in
+                 Bytes.blit_string subst 0 b b_pos len;
+                 b_pos)
+         len segments);
+    Bytes.unsafe_to_string b)
 
 let escape_utf8_string ?special_char =
   escape_string (escape_utf8_formatter ?special_char)
@@ -238,13 +280,17 @@ let escape_ascii_string ?special_char =
 let quote_utf8_string s =
   Printf.sprintf "\"%s\""
     (escape_utf8_string
-       ~special_char:(function "'" -> false | x -> utf8_special_char x)
+       ~special_char:(fun s pos len ->
+         if s.[pos] = '\'' && len = 1 then false
+         else utf8_special_char s pos len)
        s)
 
 let quote_ascii_string s =
   Printf.sprintf "\"%s\""
     (escape_ascii_string
-       ~special_char:(function "'" -> false | x -> ascii_special_char x)
+       ~special_char:(fun s pos len ->
+         if s.[pos] = '\'' && len = 1 then false
+         else ascii_special_char s pos len)
        s)
 
 let quote_string s = try quote_utf8_string s with _ -> quote_ascii_string s
@@ -397,6 +443,22 @@ let which_opt ~path s = try Some (which ~path s) with Not_found -> None
 external timezone : unit -> int = "liquidsoap_get_timezone"
   [@@noalloc]
 
+external timezone_by_name : unit -> string * string
+  = "liquidsoap_get_timezone_by_name"
+
+(* Same as [Unix.mktime] but honnors [isdst] *)
+type tm = {
+  tm_sec : int;
+  tm_min : int;
+  tm_hour : int;
+  tm_mday : int;
+  tm_mon : int;
+  tm_year : int;
+  tm_isdst : bool option;
+}
+
+external mktime : tm -> float = "liquidsoap_mktime"
+
 let string_of_timezone tz =
   (* TODO: not sure about why we need this... *)
   let tz = -tz in
@@ -837,3 +899,29 @@ let self_sync_type sources =
               | _ -> (`Dynamic, None))
           (`Static, None)
           sources))
+
+let string_of_pcre_error =
+  Pcre.(
+    function
+    | Partial -> "String only matched the pattern partially"
+    | BadPartial ->
+        "Pattern contains items that cannot be used together with partial \
+         matching."
+    | BadPattern (msg, pos) ->
+        Printf.sprintf "Malformed regular expression. Error: %s, position: %i"
+          msg pos
+    | BadUTF8 -> "UTF8 string being matched is invalid"
+    | BadUTF8Offset -> "A UTF8 string being matched with offset is invalid."
+    | MatchLimit ->
+        "Maximum allowed number of match attempts with backtracking or \
+         recursion is reached during matching."
+    | RecursionLimit -> "Maximum allowed number of recursion reached"
+    | WorkspaceSize -> "Provided workspace array is too small"
+    | InternalError msg -> Printf.sprintf "Internal error: %s" msg)
+
+let () =
+  Printexc.register_printer
+    Pcre.(
+      function
+      | Error err -> Some (Printf.sprintf "Pcre(%s)" (string_of_pcre_error err))
+      | _ -> None)
